<?php

namespace Wrm\Civserv2\Controller;

/* * *************************************************************
 *
 *  Copyright notice
 *
 *  (c) 2016 Christian Sauer <sauer@werkraum-media.de>, werkraum.media
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * OrganisationController
 */
class OrganisationController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

    /**
     * organisationRepository
     *
     * @var \Wrm\Civserv2\Domain\Repository\OrganisationRepository
     * @inject
     */
    protected $organisationRepository = NULL;

    /**
     * serviceRepository
     *
     * @var \Wrm\Civserv2\Domain\Repository\ServiceRepository
     * @inject
     */
    protected $serviceRepository = NULL;

    protected function initializeAction() {
        if (empty($this->settings['showOrganisationPID'])) {
            $this->settings['showOrganisationPID'] = intval($GLOBALS['TSFE']->id);
        }
        if (empty($this->settings['showServicePID'])) {
            $this->settings['showServicePID'] = intval($GLOBALS['TSFE']->id);
        }
        if (empty($this->settings['showFormPID'])) {
            $this->settings['showFormPID'] = intval($GLOBALS['TSFE']->id);
        }
    }

    /**
     * action list
     *
     * @return void
     */
    public function listAction() {
        $organisations = array();

        if ($this->settings['mode'] && $this->settings['mode'] == 'tree') {
            if ($this->settings['rootNode']) {
                // organisations for tree display with a given root
                $rootOrg = $this->organisationRepository->findByUid($this->settings['rootNode']);
                if ($rootOrg) {
                    $organisations = $this->organisationRepository->getTreeFor($rootOrg);
                } else {
                    $message = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Core\Messaging\FlashMessage::class, 'Invalid root node has been definded for the tree.', 'Error', \TYPO3\CMS\Core\Messaging\FlashMessage::ERROR
                    );
                    $flashMessageService = $this->objectManager->get(\TYPO3\CMS\Core\Messaging\FlashMessageService::class);
                    $messageQueue = $flashMessageService->getMessageQueueByIdentifier();
                    $messageQueue->addMessage($message);
                }
            } else {
                $message = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Core\Messaging\FlashMessage::class, 'No root node has been definded for the tree.', 'Error', \TYPO3\CMS\Core\Messaging\FlashMessage::ERROR
                );
                $flashMessageService = $this->objectManager->get(\TYPO3\CMS\Core\Messaging\FlashMessageService::class);
                $messageQueue = $flashMessageService->getMessageQueueByIdentifier();
                $messageQueue->addMessage($message);
            }
        } else {
            // organisations for A-Z display
            $res = $this->organisationRepository->findAll();

            foreach ($res as $organisation) {
                if (!empty($organisation->getOrName())) {
                    $organisations[] = array('name' => $organisation->getOrName(), 'organisation' => $organisation);
                }
            }
        }
        // sort alphabetically while also considering german umlauts
        usort($organisations, function($a, $b) {
            if ($a['name'] == $b['name'])
                return 0;
            $umlaute = Array("ä", "ö", "ü", "Ä", "Ö", "Ü", "ß", "é");
            $replace = Array("ae", "oe", "ue", "Ae", "Oe", "Ue", "ss", "e");
            $nameA = str_replace($umlaute, $replace, $a['name']);
            $nameB = str_replace($umlaute, $replace, $b['name']);

            return strnatcmp(ucfirst($nameA), ucfirst($nameB)); 
        });

        $this->view->assign('organisations', $organisations);
    }

    /**
     * action show
     *
     * @param \Wrm\Civserv2\Domain\Model\Organisation $organisation
     * @ignorevalidation $organisation
     * @return void
     */
    public function showAction(\Wrm\Civserv2\Domain\Model\Organisation $organisation = null) {
        if ($organisation) {
            $services = $this->serviceRepository->findBySvOrganisation($organisation);
            $subOrganisations = $this->organisationRepository->findSubOrganisation($organisation);

            $this->view->assign('organisation', $organisation);
            $this->view->assign('services', $services);
            $this->view->assign('subs', $subOrganisations);
        }
    }

}
