<?php

namespace Wrm\Civserv2\Controller;

/* * *************************************************************
 *
 *  Copyright notice
 *
 *  (c) 2016 Christian Sauer <sauer@werkraum-media.de>, werkraum.media
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * ServiceController
 */
class ServiceController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

    /**
     * serviceRepository
     *
     * @var \Wrm\Civserv2\Domain\Repository\ServiceRepository
     * @inject
     */
    protected $serviceRepository = NULL;

    /**
     * AccesslogRepository
     *
     * @var \Wrm\Civserv2\Domain\Repository\AccesslogRepository
     * @inject
     */
    protected $accesslogRepository = NULL;

    /**
     * EmployeePositionRepository
     *
     * @var \Wrm\Civserv2\Domain\Repository\EmployeePositionRepository
     * @inject
     */
    protected $employeePositionRepository = NULL;

    protected function initializeAction() {
        if (empty($this->settings['showOrganisationPID'])) {
            $this->settings['showOrganisationPID'] = intval($GLOBALS['TSFE']->id);
        }
        if (empty($this->settings['showServicePID'])) {
            $this->settings['showServicePID'] = intval($GLOBALS['TSFE']->id);
        }
        if (empty($this->settings['showFormPID'])) {
            $this->settings['showFormPID'] = intval($GLOBALS['TSFE']->id);
        }
        if (empty($this->settings['contactPID'])) {
            $this->settings['contactPID'] = intval($GLOBALS['TSFE']->id);
        }
    }

    /**
     * action list
     *
     * @return void
     */
    public function listAction() {
        // generate list from service names & synonyms
        $services = $this->serviceRepository->findAll();
        $res = array();
        foreach ($services as $service) {
            if (!empty($service->getSvName())) {
                $res[] = array('name' => $service->getSvName(), 'service' => $service);
                if ($service->getSvSynonym1())
                    $res[] = array('name' => $service->getSvSynonym1() . ' (=' . $service->getSvName() . ')', 'service' => $service);
                if ($service->getSvSynonym2())
                    $res[] = array('name' => $service->getSvSynonym2() . ' (=' . $service->getSvName() . ')', 'service' => $service);
                if ($service->getSvSynonym3())
                    $res[] = array('name' => $service->getSvSynonym3() . ' (=' . $service->getSvName() . ')', 'service' => $service);
            }
        }
        
        // sort alphabetically while also considering german umlauts
        usort($res, function($a, $b) {
            if ($a['name'] == $b['name'])
                return 0;
            $umlaute = Array("ä", "ö", "ü", "Ä", "Ö", "Ü", "ß", "é");
            $replace = Array("ae", "oe", "ue", "Ae", "Oe", "Ue", "ss", "e");
            $nameA = str_replace($umlaute, $replace, $a['name']);
            $nameB = str_replace($umlaute, $replace, $b['name']);

            return strnatcmp(ucfirst($nameA), ucfirst($nameB)); 
        });

        $this->view->assign('services', $res);
        $this->view->assign('total', count($res));
    }

    /**
     * action show
     *
     * @param \Wrm\Civserv2\Domain\Model\Service $service
     * @return void
     */
    public function showAction(\Wrm\Civserv2\Domain\Model\Service $service) {
        /* $positions = array();
          foreach($service->getSvPosition() as $servicePosition){
          $positions[] = $servicePosition->getPosition();
          }
          $positions =
          $employeePositions = $this->employeePositionRepository->findByPosition($service->getSvPosition()->getPosition()); */
        $this->accesslogRepository->log($service, $this->settings['logInterval']);

        $this->view->assign('service', $service);
        //$this->view->assign('employees', $employeePositions);
    }

    /**
     * action toplist
     * returns the most viewed services
     *
     * @return void
     */
    public function toplistAction() {
        $services = $this->accesslogRepository->findMostViewed($this->settings['mostViewedCount']);

        $this->view->assign('services', $services);
    }

}
