<?php

namespace Wrm\Civserv2\Domain\Repository;

/* * *************************************************************
 *
 *  Copyright notice
 *
 *  (c) 2016 Christian Sauer <sauer@werkraum-media.de>, werkraum.media
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * The repository for Accesslogs
 */
class AccesslogRepository extends \TYPO3\CMS\Extbase\Persistence\Repository {

    /**
     * action log
     *
     * @param \Wrm\Civserv2\Domain\Model\Service $service
     * @param integer $logInterval
     * @return void
     */
    public function log(\Wrm\Civserv2\Domain\Model\Service $service, $logInterval) {
        $remoteAddr = long2ip(ip2long($_SERVER['REMOTE_ADDR']));

        $query = $this->createQuery();
        $constraint[] = $query->equals('alServiceUid', $service);
        $constraint[] = $query->equals('remoteAddr', $remoteAddr);
        $constraint[] = $query->greaterThan('tstamp', time() - ($logInterval * 60));
        $query->matching($query->logicalAnd($constraint));
        $res = $query->execute();
        $count = $res->count();

        if ($count == 0) {
            $al = $this->objectManager->get('Wrm\Civserv2\Domain\Model\Accesslog');
            $al->setAlServiceUid($service);
            $al->setAlNumber(1);
            $al->setRemoteAddr($remoteAddr);
            $this->add($al);
            $persistenceManager = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Persistence\\Generic\\PersistenceManager');
            $persistenceManager->persistAll();

            $this->aggregateLog($logInterval);
        }
    }

    /**
     * action aggregateLog
     *
     * @param integer $logInterval
     * @return void
     */
    public function aggregateLog($logInterval) {
        $query = $this->createQuery();
        $constraint[] = $query->lessThan('tstamp', time() - ($logInterval * 86400));
        $constraint[] = $query->equals('remoteAddr', '');
        $query->matching($query->logicalOr($constraint));
        $res = $query->execute();
        $serviceRep = $this->objectManager->get('Wrm\Civserv2\Domain\Repository\ServiceRepository');

        $aggr = array();
        foreach ($res as $single) {
            if ($single && $single->getUid() && $single->getAlServiceUid()) {
                if (isset($aggr[$single->getAlServiceUid()->getUid()])) {
                    $aggr[$single->getAlServiceUid()->getUid()] = $aggr[$single->getAlServiceUid()->getUid()] + $single->getAlNumber();
                } else {
                    $aggr[$single->getAlServiceUid()->getUid()] = $single->getAlNumber();
                }
                $this->remove($single);
            }
        }

        foreach ($aggr as $key => $val) {
            $service = $serviceRep->findByUid($key);
            $al = $this->objectManager->get('Wrm\Civserv2\Domain\Model\Accesslog');
            $al->setAlServiceUid($service);
            $al->setAlNumber($val);
            $this->add($al);
        }

        $GLOBALS['TYPO3_DB']->exec_DELETEquery('tx_civserv2_domain_model_accesslog', "deleted=1");
    }

    /**
     * action findMostViewed
     *
     * @param integer $mostViewedCount
     * @return void
     */
    public function findMostViewed($mostViewedCount) {
        $res = $this->findAll();
        $services = array();

        // TODO: Performance is quite bad for this!
        foreach ($res as $single) {
            if (isset($services[$single->getAlServiceUid()->getUid()])) {
                $services[$single->getAlServiceUid()->getUid()]['count'] = $services[$single->getAlServiceUid()->getUid()]['count'] + $single->getAlNumber();
            } else {
                $services[$single->getAlServiceUid()->getUid()] = array('service' => $single->getAlServiceUid(), 'count' => $single->getAlNumber());
            }
        }

        usort($services, function($a, $b) {
            if ($a['count'] == $b['count'])
                return 0;
            return $a['count'] > $b['count'] ? -1 : 1;
        });

        return array_slice($services, 0, $mostViewedCount);
    }

}
