<?php

namespace Wrm\Civserv2\Hooks;

/* * *************************************************************
 *  Copyright notice
 *
 *  (c) 2017 Christian Sauer <sauer@werkraum-media.de>, werkraum.media
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * RealUrl AutoConfiguration
 */
class RealUrlAutoConfiguration {

    /**
     * Generates additional RealURL configuration and merges it with provided configuration
     *
     * @param array $params Default configuration
     * @param tx_realurl_autoconfgen $pObj parent object
     * @return array Updated configuration
     */
    public function addExtConfig($params, &$pObj) {
        return array_merge_recursive($params['config'], [
            //'disablePathCache' => true,
            'cache' => [
                'banUrlsRegExp' => '/tx_civserv2_pi1|gclid|tx_solr|tx_indexed_search|(?:^|\?|&)q=/',
            ],
            'postVarSets' => [
                '_DEFAULT' => [
                    'information' => array(
                        array(
                            'GETvar' => 'tx_civserv2_pi1[controller]',
                            'valueMap' => array(
                                'anliegen' => 'Service',
                                'amt' => 'Organisation',
                                'formular' => 'Form',
                                'kontakt' => 'Contact'
                            ),
                            'noMatch' => 'bypass',
                        ),
                        array(
                            'GETvar' => 'tx_civserv2_pi1[action]',
                            'valueMap' => array(
                                'detail' => 'show',
                            ),
                            'noMatch' => 'bypass',
                        ),
                        array(
                            'GETvar' => 'tx_civserv2_pi1[service]',
                            'lookUpTable' => array(
                                'table' => 'tx_civserv2_domain_model_service',
                                'id_field' => 'uid',
                                'alias_field' => 'CONCAT(sv_name,"-",uid)',
                                'addWhereClause' => ' AND NOT deleted',
                                'autoUpdate' => 1,
                                'expireDays' => 1,
                                'useUniqueCache' => 1,
                                'useUniqueCache_conf' => array(
                                    'strtolower' => 1,
                                    'spaceCharacter' => '-',
                                ),
                            ),
                        ),
                        array(
                            'GETvar' => 'tx_civserv2_pi1[organisation]',
                            'lookUpTable' => array(
                                'table' => 'tx_civserv2_domain_model_organisation',
                                'id_field' => 'uid',
                                'alias_field' => 'CONCAT(or_name,"-",uid)',
                                'addWhereClause' => ' AND NOT deleted',
                                'autoUpdate' => 1,
                                'expireDays' => 1,
                                'useUniqueCache' => 1,
                                'useUniqueCache_conf' => array(
                                    'strtolower' => 1,
                                    'spaceCharacter' => '-',
                                ),
                            ),
                        ),
                        array(
                            'GETvar' => 'tx_civserv2_pi1[emPosition]',
                            'lookUpTable' => array(
                                'table' => 'tx_civserv2_domain_model_employeeposition',
                                'id_field' => 'uid',
                                'alias_field' => 'position',
                                'useUniqueCache' => 1,
                                'useUniqueCache_conf' => array(
                                    'strtolower' => 1,
                                    'spaceCharacter' => '-',
                                )
                            )
                        ),
                        array(
                            'GETvar' => 'tx_civserv2_pi1[form]',
                            'lookUpTable' => array(
                                'table' => 'tx_civserv2_domain_model_form',
                                'id_field' => 'uid',
                                'alias_field' => 'CONCAT(fo_name,"-",uid)',
                                'addWhereClause' => ' AND NOT deleted',
                                'autoUpdate' => 1,
                                'expireDays' => 1,
                                'useUniqueCache' => 1,
                                'useUniqueCache_conf' => array(
                                    'strtolower' => 1,
                                    'spaceCharacter' => '-',
                                ),
                            )
                        )
                    ),
                    'az' => array(
                        array(
                            'GETvar' => 'tx_civserv2_pi1[@widget_0][currentChar]',
                        //'noMatch' => 'bypass'
                        )
                    ),
                    'pag' => array(
                        array(
                            'GETvar' => 'tx_civserv2_pi1[@widget_1][currentPage]',
                        //'noMatch' => 'bypass'
                        )
                    ),
                    'kontakt' => array(
                        array(
                            'GETvar' => 'tx_civserv2_pi1[emPosition]',
                            'lookUpTable' => array(
                                'table' => 'tx_civserv2_domain_model_employeeposition',
                                'id_field' => 'uid',
                                'alias_field' => 'ep_label',
                                'addWhereClause' => ' AND NOT deleted',
                                'autoUpdate' => 1,
                                'expireDays' => 1,
                                'useUniqueCache' => 1,
                                'useUniqueCache_conf' => array(
                                    'strtolower' => 1,
                                    'spaceCharacter' => '-',
                                ),
                            ),
                        ),
                    ),
                ]
            ],
            'fileName' => [
                'defaultToHTMLsuffixOnPrev' => 1,
                'acceptHTMLsuffix' => 1,
                'index' => [
                ],
            ],
                ]
        );
    }

}
