<?php

namespace Wrm\Civserv2\Service;

/* * *************************************************************
 *
 *  Copyright notice
 *
 *  (c) 2016 Christian Sauer <sauer@werkraum-media.de>, werkraum.media
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 * ************************************************************* */

/**
 * Class MailService
 */
class MailService {

    /**
     * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
     * @inject
     */
    protected $configurationManager;
    
    /**
     * employeePositionRepository
     *
     * @var \Wrm\Civserv2\Domain\Repository\EmployeePositionRepository
     * @inject
     */
    protected $employeePositionRepository;

    /**
     * @param \Wrm\Civserv2\Domain\Model\Contact $contact
     * @param string $defaultFromEmail
     * @return boolean
     * @throws \TYPO3\CMS\Extbase\Configuration\Exception
     */
    public function sendEmail(\Wrm\Civserv2\Domain\Model\Contact $contact) {
        $settings = $this->configurationManager->getConfiguration(\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS);
        $emPosition = $contact->getEmPosition();
        $fromEmail = TRUE === (boolean) !empty($contact->getEmail()) ? $contact->getEmail() : $settings['fromEmail'];
        $fromName = $contact->getFirstname().' '.$contact->getSurname();
        $charset = 'utf8';
        
        $subject = $contact->getSubject();
        $toEmail = !empty($emPosition->getEpEmail()) ? $emPosition->getEpEmail() : $emPosition->getEmployee()->getEmEmail();
        $toName = $emPosition->getEmployee()->getEmFirstname().' '.$emPosition->getEmployee()->getEmName();
        
        $htmlView = $this->getView('Contact', 'html');
        $htmlView->assign('submission', $contact);
        $htmlView->assign('charset', $charset);
        $htmlView->assign('title', $subject);
        $htmlBody = $htmlView->render();
        $plainView = $this->getView('Contact', 'txt');
        $plainView->assign('submission', $contact);
        $plainView->assign('charset', $charset);
        $plainView->assign('title', $subject);
        $plainBody = $plainView->render();
        /** @var \TYPO3\CMS\Core\Mail\MailMessage $message */
        $message = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Core\\Mail\\MailMessage');
        $message->setTo(array($toEmail => $toName));
        $message->setFrom(array($fromEmail => $fromName));
        $message->setSubject($subject);
        $message->setCharset($charset);
        $message->setBody($htmlBody, 'text/html');
        $message->addPart($plainBody, 'text/plain');
        $message->send();
               
        return $message->isSent();
    }

    /**
     * @param string $templateName
     * @param string $format
     * @return \TYPO3\CMS\Fluid\View\StandaloneView
     */
    protected function getView($templateName, $format = 'html') {
        /** @var \TYPO3\CMS\Fluid\View\StandaloneView $view */
        $view = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Fluid\\View\\StandaloneView');
        $view->setFormat($format);
        $view->getRequest()->setControllerExtensionName('civserv2');
        $extbaseFrameworkConfiguration = $this->configurationManager->getConfiguration(\TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK);
        $templateRootPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName($extbaseFrameworkConfiguration['view']['templateRootPaths'][0]);
        $layoutRootPath = \TYPO3\CMS\Core\Utility\GeneralUtility::getFileAbsFileName($extbaseFrameworkConfiguration['view']['layoutRootPaths'][0]);
        $templatePathAndFilename = $templateRootPath . 'Email/' . $templateName . '.' . $format;
        $view->setTemplatePathAndFilename($templatePathAndFilename);
        $view->setLayoutRootPaths(array($layoutRootPath));
        return $view;
    }

}
