<?php

namespace Wrm\Civserv2\Tests\Unit\Domain\Model;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2016 Christian Sauer <sauer@werkraum-media.de>, werkraum.media
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * Test case for class \Wrm\Civserv2\Domain\Model\ExternalService.
 *
 * @copyright Copyright belongs to the respective authors
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 * @author Christian Sauer <sauer@werkraum-media.de>
 */
class ExternalServiceTest extends \TYPO3\CMS\Core\Tests\UnitTestCase
{
	/**
	 * @var \Wrm\Civserv2\Domain\Model\ExternalService
	 */
	protected $subject = NULL;

	public function setUp()
	{
		$this->subject = new \Wrm\Civserv2\Domain\Model\ExternalService();
	}

	public function tearDown()
	{
		unset($this->subject);
	}

	/**
	 * @test
	 */
	public function getEsNameReturnsInitialValueForString()
	{
		$this->assertSame(
			'',
			$this->subject->getEsName()
		);
	}

	/**
	 * @test
	 */
	public function setEsNameForStringSetsEsName()
	{
		$this->subject->setEsName('Conceived at T3CON10');

		$this->assertAttributeEquals(
			'Conceived at T3CON10',
			'esName',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getEsNavigationReturnsInitialValueForNavigation()
	{
		$newObjectStorage = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		$this->assertEquals(
			$newObjectStorage,
			$this->subject->getEsNavigation()
		);
	}

	/**
	 * @test
	 */
	public function setEsNavigationForObjectStorageContainingNavigationSetsEsNavigation()
	{
		$esNavigation = new \Wrm\Civserv2\Domain\Model\Navigation();
		$objectStorageHoldingExactlyOneEsNavigation = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		$objectStorageHoldingExactlyOneEsNavigation->attach($esNavigation);
		$this->subject->setEsNavigation($objectStorageHoldingExactlyOneEsNavigation);

		$this->assertAttributeEquals(
			$objectStorageHoldingExactlyOneEsNavigation,
			'esNavigation',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function addEsNavigationToObjectStorageHoldingEsNavigation()
	{
		$esNavigation = new \Wrm\Civserv2\Domain\Model\Navigation();
		$esNavigationObjectStorageMock = $this->getMock('TYPO3\\CMS\\Extbase\\Persistence\\ObjectStorage', array('attach'), array(), '', FALSE);
		$esNavigationObjectStorageMock->expects($this->once())->method('attach')->with($this->equalTo($esNavigation));
		$this->inject($this->subject, 'esNavigation', $esNavigationObjectStorageMock);

		$this->subject->addEsNavigation($esNavigation);
	}

	/**
	 * @test
	 */
	public function removeEsNavigationFromObjectStorageHoldingEsNavigation()
	{
		$esNavigation = new \Wrm\Civserv2\Domain\Model\Navigation();
		$esNavigationObjectStorageMock = $this->getMock('TYPO3\\CMS\\Extbase\\Persistence\\ObjectStorage', array('detach'), array(), '', FALSE);
		$esNavigationObjectStorageMock->expects($this->once())->method('detach')->with($this->equalTo($esNavigation));
		$this->inject($this->subject, 'esNavigation', $esNavigationObjectStorageMock);

		$this->subject->removeEsNavigation($esNavigation);

	}

	/**
	 * @test
	 */
	public function getEsExternalServiceReturnsInitialValueForService()
	{
		$newObjectStorage = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		$this->assertEquals(
			$newObjectStorage,
			$this->subject->getEsExternalService()
		);
	}

	/**
	 * @test
	 */
	public function setEsExternalServiceForObjectStorageContainingServiceSetsEsExternalService()
	{
		$esExternalService = new \Wrm\Civserv2\Domain\Model\Service();
		$objectStorageHoldingExactlyOneEsExternalService = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		$objectStorageHoldingExactlyOneEsExternalService->attach($esExternalService);
		$this->subject->setEsExternalService($objectStorageHoldingExactlyOneEsExternalService);

		$this->assertAttributeEquals(
			$objectStorageHoldingExactlyOneEsExternalService,
			'esExternalService',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function addEsExternalServiceToObjectStorageHoldingEsExternalService()
	{
		$esExternalService = new \Wrm\Civserv2\Domain\Model\Service();
		$esExternalServiceObjectStorageMock = $this->getMock('TYPO3\\CMS\\Extbase\\Persistence\\ObjectStorage', array('attach'), array(), '', FALSE);
		$esExternalServiceObjectStorageMock->expects($this->once())->method('attach')->with($this->equalTo($esExternalService));
		$this->inject($this->subject, 'esExternalService', $esExternalServiceObjectStorageMock);

		$this->subject->addEsExternalService($esExternalService);
	}

	/**
	 * @test
	 */
	public function removeEsExternalServiceFromObjectStorageHoldingEsExternalService()
	{
		$esExternalService = new \Wrm\Civserv2\Domain\Model\Service();
		$esExternalServiceObjectStorageMock = $this->getMock('TYPO3\\CMS\\Extbase\\Persistence\\ObjectStorage', array('detach'), array(), '', FALSE);
		$esExternalServiceObjectStorageMock->expects($this->once())->method('detach')->with($this->equalTo($esExternalService));
		$this->inject($this->subject, 'esExternalService', $esExternalServiceObjectStorageMock);

		$this->subject->removeEsExternalService($esExternalService);

	}
}
