<?php

namespace Wrm\Civserv2\Tests\Unit\Domain\Model;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2016 Christian Sauer <sauer@werkraum-media.de>, werkraum.media
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

/**
 * Test case for class \Wrm\Civserv2\Domain\Model\Position.
 *
 * @copyright Copyright belongs to the respective authors
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 * @author Christian Sauer <sauer@werkraum-media.de>
 */
class PositionTest extends \TYPO3\CMS\Core\Tests\UnitTestCase
{
	/**
	 * @var \Wrm\Civserv2\Domain\Model\Position
	 */
	protected $subject = NULL;

	public function setUp()
	{
		$this->subject = new \Wrm\Civserv2\Domain\Model\Position();
	}

	public function tearDown()
	{
		unset($this->subject);
	}

	/**
	 * @test
	 */
	public function getPoNameReturnsInitialValueForString()
	{
		$this->assertSame(
			'',
			$this->subject->getPoName()
		);
	}

	/**
	 * @test
	 */
	public function setPoNameForStringSetsPoName()
	{
		$this->subject->setPoName('Conceived at T3CON10');

		$this->assertAttributeEquals(
			'Conceived at T3CON10',
			'poName',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getPoNiceNameReturnsInitialValueForString()
	{
		$this->assertSame(
			'',
			$this->subject->getPoNiceName()
		);
	}

	/**
	 * @test
	 */
	public function setPoNiceNameForStringSetsPoNiceName()
	{
		$this->subject->setPoNiceName('Conceived at T3CON10');

		$this->assertAttributeEquals(
			'Conceived at T3CON10',
			'poNiceName',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getPoNumberReturnsInitialValueForString()
	{
		$this->assertSame(
			'',
			$this->subject->getPoNumber()
		);
	}

	/**
	 * @test
	 */
	public function setPoNumberForStringSetsPoNumber()
	{
		$this->subject->setPoNumber('Conceived at T3CON10');

		$this->assertAttributeEquals(
			'Conceived at T3CON10',
			'poNumber',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getPoDescrReturnsInitialValueForString()
	{
		$this->assertSame(
			'',
			$this->subject->getPoDescr()
		);
	}

	/**
	 * @test
	 */
	public function setPoDescrForStringSetsPoDescr()
	{
		$this->subject->setPoDescr('Conceived at T3CON10');

		$this->assertAttributeEquals(
			'Conceived at T3CON10',
			'poDescr',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function getPoOrganisationReturnsInitialValueForOrganisation()
	{
		$newObjectStorage = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		$this->assertEquals(
			$newObjectStorage,
			$this->subject->getPoOrganisation()
		);
	}

	/**
	 * @test
	 */
	public function setPoOrganisationForObjectStorageContainingOrganisationSetsPoOrganisation()
	{
		$poOrganisation = new \Wrm\Civserv2\Domain\Model\Organisation();
		$objectStorageHoldingExactlyOnePoOrganisation = new \TYPO3\CMS\Extbase\Persistence\ObjectStorage();
		$objectStorageHoldingExactlyOnePoOrganisation->attach($poOrganisation);
		$this->subject->setPoOrganisation($objectStorageHoldingExactlyOnePoOrganisation);

		$this->assertAttributeEquals(
			$objectStorageHoldingExactlyOnePoOrganisation,
			'poOrganisation',
			$this->subject
		);
	}

	/**
	 * @test
	 */
	public function addPoOrganisationToObjectStorageHoldingPoOrganisation()
	{
		$poOrganisation = new \Wrm\Civserv2\Domain\Model\Organisation();
		$poOrganisationObjectStorageMock = $this->getMock('TYPO3\\CMS\\Extbase\\Persistence\\ObjectStorage', array('attach'), array(), '', FALSE);
		$poOrganisationObjectStorageMock->expects($this->once())->method('attach')->with($this->equalTo($poOrganisation));
		$this->inject($this->subject, 'poOrganisation', $poOrganisationObjectStorageMock);

		$this->subject->addPoOrganisation($poOrganisation);
	}

	/**
	 * @test
	 */
	public function removePoOrganisationFromObjectStorageHoldingPoOrganisation()
	{
		$poOrganisation = new \Wrm\Civserv2\Domain\Model\Organisation();
		$poOrganisationObjectStorageMock = $this->getMock('TYPO3\\CMS\\Extbase\\Persistence\\ObjectStorage', array('detach'), array(), '', FALSE);
		$poOrganisationObjectStorageMock->expects($this->once())->method('detach')->with($this->equalTo($poOrganisation));
		$this->inject($this->subject, 'poOrganisation', $poOrganisationObjectStorageMock);

		$this->subject->removePoOrganisation($poOrganisation);

	}
}
